@@ -173,14 +173,17 @@ class Context(object):
             default_map = parent.default_map.get(info_name)
         self.default_map = default_map
 
-        #: This is similar to :attr:`invoked_subcommand` but it holds a
-        #: list of multiple subcommand names.  This is useful if multi command
-        #: chaining is enabled.
+        #: This flag indicates if a subcommand is going to be executed. A
+        #: group callback can use this information to figure out if it's
+        #: being executed directly or because the execution flow passes
+        #: onwards to a subcommand. By default it's None, but it can be
+        #: the name of the subcommand to execute.
         #:
-        #: This property and :attr:`invoked_subcommand` are synchronized.
-        #:
-        #: .. versionadded:: 3.0
-        self.invoked_subcommands = []
+        #: If chaining is enabled this will be set to ``'*'`` in case
+        #: any commands are executed.  It is however not possible to
+        #: figure out which ones.  If you require this knowledge you
+        #: should use a :func:`resultcallback`.
+        self.invoked_subcommand = None
 
         if terminal_width is None and parent is not None:
             terminal_width = parent.terminal_width
@@ -248,28 +251,24 @@ class Context(object):
         if self._depth == 0:
             self.close()
 
-    def _get_invoked_subcommand(self):
-        """This flag indicates if a subcommand is going to be executed.  A
-        group callback can use this information to figure out if it's
-        being executed directly or because the execution flow passes
-        onwards to a subcommand.  By default it's `None`, but it can be
-        the name of the subcommand to execute.
-
-        If chaining is enabled and more than one subcommand is invoked
-        then the value will be the string ``'*'``.  For chaining you
-        should be using :attr:`invoked_subcommands` instead.
-        """
-        if len(self.invoked_subcommands) == 1:
-            return self.invoked_subcommands[0]
-        if len(self.invoked_subcommands) > 1:
-            return '*'
-    def _set_invoked_subcommand(self, value):
-        if value is not None:
-            value = [value]
-        self.invoked_subcommands = value
-    invoked_subcommand = property(_get_invoked_subcommand,
-                                  _set_invoked_subcommand)
-    del _get_invoked_subcommand, _set_invoked_subcommand
+    def _get_invoked_subcommands(self):
+        from warnings import warn
+        warn(Warning('This API does not work properly and has been largely '
+                     'removed in Click 3.2 to fix a regression the '
+                     'introduction of this API caused.  Consult the '
+                     'upgrade documentation for more information.  For '
+                     'more information about this see '
+                     'http://click.pocoo.org/upgrading/#upgrading-to-3.2'),
+             stacklevel=2)
+        if self.invoked_subcommand is None:
+            return []
+        return [self.invoked_subcommand]
+    def _set_invoked_subcommands(self, value):
+        self.invoked_subcommand = \
+            len(value) > 1 and '*' or value and value[0] or None
+    invoked_subcommands = property(_get_invoked_subcommands,
+                                   _set_invoked_subcommands)
+    del _get_invoked_subcommands, _set_invoked_subcommands
 
     def make_formatter(self):
         """Creates the formatter for the help and usage output."""
@@ -369,26 +368,65 @@ class Context(object):
         return self.command.get_help(self)
 
     def invoke(*args, **kwargs):
-        """Invokes a command callback in exactly the way it expects.
+        """Invokes a command callback in exactly the way it expects.  There
+        are two ways to invoke this method:
+
+        1.  the first argument can be a callback and all other arguments and
+            keyword arguments are forwarded directly to the function.
+        2.  the first argument is a click command object.  In that case all
+            arguments are forwarded as well but proper click parameters
+            (options and click arguments) must be keyword arguments and Click
+            will fill in defaults.
+
+        Note that before Click 3.2 keyword arguments were not properly filled
+        in against the intention of this code and no context was created.  For
+        more information about this change and why it was done in a bugfix
+        release see :ref:`upgrade-to-3.2`.
         """
         self, callback = args[:2]
+        ctx = self
+
+        # This is just to improve the error message in cases where old
+        # code incorrectly invoked this method.  This will eventually be
+        # removed.
+        injected_arguments = False
 
         # It's also possible to invoke another command which might or
-        # might not have a callback.
+        # might not have a callback.  In that case we also fill
+        # in defaults and make a new context for this command.
         if isinstance(callback, Command):
-            callback = callback.callback
+            other_cmd = callback
+            callback = other_cmd.callback
+            ctx = Context(other_cmd, info_name=other_cmd.name, parent=self)
             if callback is None:
                 raise TypeError('The given command does not have a '
                                 'callback that can be invoked.')
 
+            for param in other_cmd.params:
+                if param.name not in kwargs and param.expose_value:
+                    kwargs[param.name] = param.get_default(ctx)
+                    injected_arguments = True
+
         args = args[2:]
         if getattr(callback, '__click_pass_context__', False):
-            args = (self,) + args
+            args = (ctx,) + args
         with augment_usage_errors(self):
-            return callback(*args, **kwargs)
+            try:
+                with ctx:
+                    return callback(*args, **kwargs)
+            except TypeError as e:
+                if not injected_arguments:
+                    raise
+                if 'got multiple values for keyword argument' in str(e):
+                    raise RuntimeError(
+                        'You called .invoke() on the context with a command '
+                        'but provided parameters as positional arguments.  '
+                        'This is not supported but sometimes worked by chance '
+                        'in older versions of Click.  To fix this see '
+                        'http://click.pocoo.org/upgrading/#upgrading-to-3.2')
 
     def forward(*args, **kwargs):
-        """Similar to :meth:`forward` but fills in default keyword
+        """Similar to :meth:`invoke` but fills in default keyword
         arguments from the current context if the other command expects
         it.  This cannot invoke callbacks directly, only other commands.
         """
@@ -400,8 +438,7 @@ class Context(object):
             raise TypeError('Callback is not a command.')
 
         for param in self.params:
-            if param in self.params and \
-               param not in kwargs:
+            if param not in kwargs:
                 kwargs[param] = self.params[param]
 
         return self.invoke(cmd, **kwargs)
@@ -764,6 +801,8 @@ class MultiCommand(Command):
                   is enabled.  This restricts the form of commands in that
                   they cannot have optional arguments but it allows
                   multiple commands to be chained together.
+    :param result_callback: the result callback to attach to this multi
+                            command.
     """
     allow_extra_args = True
     allow_interspersed_args = False
@@ -783,6 +822,8 @@ class MultiCommand(Command):
                 subcommand_metavar = SUBCOMMAND_METAVAR
         self.subcommand_metavar = subcommand_metavar
         self.chain = chain
+        #: The result callback that is stored.  This can be set or
+        #: overridden with the :func:`resultcallback` decorator.
         self.result_callback = result_callback
 
     def collect_usage_pieces(self, ctx):
@@ -886,15 +927,20 @@ class MultiCommand(Command):
             # Make sure the context is entered so we do not clean up
             # resources until the result processor has worked.
             with ctx:
+                cmd_name, cmd, args = self.resolve_command(ctx, args)
+                ctx.invoked_subcommand = cmd_name
                 Command.invoke(self, ctx)
-                sub_ctx = self.handle_subcommand(ctx, args)
-                ctx.invoked_subcommands = [sub_ctx.info_name]
+                sub_ctx = cmd.make_context(cmd_name, args, parent=ctx)
                 with sub_ctx:
                     return _process_result(sub_ctx.command.invoke(sub_ctx))
 
-        # Make sure the context is entered so we do not clean up
-        # resources until the result processor has worked.
+        # In chain mode we create the contexts step by step, but after the
+        # base command has been invoked.  Because at that point we do not
+        # know the subcommands yet, the invoked subcommand attribute is
+        # set to ``*`` to inform the command that subcommands are executed
+        # but nothing else.
         with ctx:
+            ctx.invoked_subcommand = args and '*' or None
             Command.invoke(self, ctx)
 
             # Otherwise we make every single context and invoke them in a
@@ -902,21 +948,20 @@ class MultiCommand(Command):
             # is the list of all invoked subcommand's results.
             contexts = []
             while args:
-                sub_ctx = self.handle_subcommand(ctx, args, allow_extra_args=True,
-                                                 allow_interspersed_args=False)
+                cmd_name, cmd, args = self.resolve_command(ctx, args)
+                sub_ctx = cmd.make_context(cmd_name, args, parent=ctx,
+                                           allow_extra_args=True,
+                                           allow_interspersed_args=False)
                 contexts.append(sub_ctx)
                 args = sub_ctx.args
 
-            # Now that we have all contexts, we can invoke them.
-            ctx.invoked_subcommands = [x.info_name for x in contexts]
-
             rv = []
             for sub_ctx in contexts:
                 with sub_ctx:
                     rv.append(sub_ctx.command.invoke(sub_ctx))
             return _process_result(rv)
 
-    def handle_subcommand(self, ctx, args, **extra):
+    def resolve_command(self, ctx, args):
         cmd_name = make_str(args[0])
         original_cmd_name = cmd_name
 
@@ -940,7 +985,7 @@ class MultiCommand(Command):
                 self.parse_args(ctx, ctx.args)
             ctx.fail('No such command "%s".' % original_cmd_name)
 
-        return cmd.make_context(cmd_name, args[1:], parent=ctx, **extra)
+        return cmd_name, cmd, args[1:]
 
     def get_command(self, ctx, cmd_name):
         """Given a context and a command name, this returns a
