@@ -1,6 +1,9 @@
+import os
 import sys
 import click
 
+import click._termui_impl
+
 
 def test_echo(runner):
     with runner.isolation() as out:
@@ -111,3 +114,32 @@ def test_prompts(runner):
     result = runner.invoke(test_no, input='n\n')
     assert not result.exception
     assert result.output == 'Foo [Y/n]: n\nno :(\n'
+
+
+def test_echo_via_pager(monkeypatch, capfd):
+    monkeypatch.setitem(os.environ, 'PAGER', 'cat')
+    monkeypatch.setattr(click._termui_impl, 'isatty', lambda x: True)
+    click.echo_via_pager('haha')
+    out, err = capfd.readouterr()
+    assert out == 'haha\n'
+
+
+def test_open_file(runner):
+    with runner.isolated_filesystem():
+        with open('hello.txt', 'w') as f:
+            f.write('Cool stuff')
+
+        @click.command()
+        @click.argument('filename')
+        def cli(filename):
+            with click.open_file(filename) as f:
+                click.echo(f.read())
+            click.echo('meep')
+
+        result = runner.invoke(cli, ['hello.txt'])
+        assert result.exception is None
+        assert result.output == 'Cool stuff\nmeep\n'
+
+        result = runner.invoke(cli, ['-'], input='foobar')
+        assert result.exception is None
+        assert result.output == 'foobar\nmeep\n'
